/*
!!! THE INFORMATION INCLUDED IN MY PACKAGE IS ONLY FOR EDUCATIONAL PURPOSE AND
!!! SHOULD NOT BE USED AGAINST ANY HOST WITHOUT PERMISSION FROM THE SYSTEM
!!! ADMINISTRATOR.
*/

// INCLUDE
#include <stdio.h>		// Generic functions
#include "winsock.h"	// Socket functions

// Current Version
#define VERSION		"0.1"

// Character for comment and newline char
// delimiter '=' in config file
#define COMCHAR		'#'
#define NLCHAR		'\n'
#define DECHAR		"="

// End of malformed string
#define ENDMSTR			"--"

// Const for Print and Network mode
#define NETMODEUDP 3		// Send strings over network udp
#define	NETMODETCP 2		// Send strings over network tcp
#define PRINTMODE	 1		// Only print strings

// Max and min of vars
#define MAXHSIZ		255+1				// Maximum hostname size
#define MAXFSIZ		1024+1		  // Maximum filename size
#define MAXRSIZ		9999999+1		// Maximum request size
#define MAXSSIZ		6+1					// Maximum subchar size
#define MAXFOPTS	5						// Maximum number of file entries

// Error handling config file
#define ENDOFFERR			15			// End of file error
#define NETREQTERR		14			// Network request error
#define INVNUMERR			13			// Invalid number error
#define DELEMERR			12			// Delemiter error
#define STEOSEQERR		11			// Steo-sequence error
#define MAXSEQERR			10			// Max-sequence error
#define STARTSEQERR		9				// Start-sequence error
#define SUBCHARERR		8				// subchar error

// Error handling parameters
#define RETPORTERR			6						// Invalid port number
#define RETFILEERR			5						// Cannot open file

// Network error handling
#define RETCONNERR			4						// Connection error
#define RETSOCKERR			3						// Return for socket error
#define RETRESVERR			2						// Error code for cannot resolve host
#define RETOK						1						// Return OK

// Static strings for error handling
// Network error handling
#define CONNERRSTR						"\nCannot connect to port !!!\n"
#define SOCKERRSTR						"\nCannot create socket !!!\n"
#define RESVERRSTR						"\nCannot resolve host !!!\n"

// Parameter error handling
#define FERRSTR								"\nCannot open file !!!\n"
#define PORTERRSTR						"\nInvalid port number !!!\n"
#define UNKERRSTR							"\nUnknown error !!!\n"
#define PARAMERRSTR						"\nWrong parameter !!!\n"

// File error handling strings
#define NETREQTERRSTR					"\nCannot find NETREQT tag !!!\n"
#define SUBCHARERRSTR					"\nCannot find SUBCHAR tag !!!\n"
#define STARTSEQERRSTR				"\nCannot find STARTSEQ tag !!!\n"
#define MAXSEQERRSTR					"\nCannot find MAXSEQ tag !!!\n"
#define STEOSEQERRSTR					"\nCannot find STEOSEQ tag !!!\n"
#define DELEMERRSTR						"\nPlease use format TAG=value in config file !!!\n"
#define	INVNUMERRSTR					"\nOnly numeric values greater than 0 allowed for"\
															" STARTSEQ, MAXSEQ, STEOSEQ, NETREQT in config file !!!\n"
#define	INVCFRRSTR						"\nInvalid config file !!!\n"
#define	STRINGERRSTR					"\nCannot create requests !!!\n"

// Print possible output macro
#define BUGOUTPUT()						fprintf(stderr, "\nCannot connect to TCP port %d\n", portnum);\
															fprintf(stderr, "Possible bug or NETREQT to low,\n");\
															fprintf(stderr, "because I cannot connect !!!\n");\
															fprintf(stderr, "\nPress a key to continue fuzzing.\n\n\n");\
															getchar();\
															fprintf(stderr, "Now I continue !!!\n\n");


// Config file strings
#define SUBCHARSTR						"SUBCHAR"
#define STARTSEQSTR						"STARTSEQ"
#define MAXSEQSTR							"MAXSEQ"
#define STEOSEQSTR						"STEOSEQ"
#define NETREQTSTR						"NETREQT"
#define MALFSTR								"MALFORMED"

// Prototypes
// Connect to hostname port portnum and send string
int ConnectWithString(char *hostname, unsigned int portnum, char *string, char *proto);

// Print usage for this tool
void PrintUsage(char *argv);

// Check for correct parameters
int CheckParams(int mode);

// Parse configuration file
int ParseFile(void);

// overstep comment and newlines
int ParseToDataComNL(FILE *file, char *str);

// overstep comments only
int ParseToDataCom(FILE *file, char *str);

// Print errors
void PrintCheckRetVal(int ret);
void PrintFileRetVal(int ret);
void PrintStringsRetVal(int ret);

// Generate and print or send strings
int GenStrings(void);

// Global vars
char *hostname;					// Hostname
char *filename;					// Config filename
unsigned int portnum;		// Port number.
int mode;								// Print or network mode.
	
	
// Struct for filename parsing
struct
{
	char subchar[MAXSSIZ];		// Character used for malformed strings
	int startseq;							// With how many chars should i start ?
	int maxseq;								// Maximum size of malformed chars
	int steoseq;							// Step steoseq each request
	int netreqt;							// Time between requests in milliseconds
} filearg;


////////////// MAIN ////////////////
int main(int argc, char **argv)
{	
	// local return value
	int ret;
		
	// Only three or one parameter allowed
	if((argc != 5) && (argc != 2))
	{
		// Usage and return error (-1)
		PrintUsage(argv[0]);
		return -1;
	}
	
	// Set mode depends on argc
	if(argc == 5)
	{
		if(!strcmp(argv[4], "TCP"))
			mode = NETMODETCP;
		else if(!strcmp(argv[4], "UDP"))
			mode = NETMODEUDP;
		else
		{
			PrintUsage(argv[0]);
			return -2;			
		}
	}
	
	else
		mode = PRINTMODE;
	
	if((mode == NETMODETCP) || (mode == NETMODEUDP))
	{
		// Save space for hostname
		hostname = malloc(MAXHSIZ);
				
		// Copy hostname to global var
		strncpy(hostname, argv[1], MAXHSIZ-1);
		portnum = atoi(argv[2]);
	}
	
	// Space for filename and copy filename string
	filename = malloc(MAXFSIZ);
	
	// Copy the filename
	if(mode == PRINTMODE)
		strncpy(filename, argv[1], MAXFSIZ-1);
	else
		strncpy(filename, argv[3], MAXFSIZ-1);
		
	// Check parameters
	ret = CheckParams(mode);
	
	// Everything correct ?
	if(ret != RETOK)
	{
		// Print error and exit
		PrintCheckRetVal(ret);
		return -2;
	}

	// Read configuration from file
	ret = ParseFile();
	
	// Print ParseFile() error messages
	if(ret != RETOK)
	{
		PrintFileRetVal(ret);
		return -3;
	}
	
	// Generate strings and send or print it
	ret = GenStrings();
	
	// Error occured ?
	if(ret != RETOK)
	{
		PrintStringsRetVal(ret);
		return -4;
	}
	
	// Free hostname and filename
	if((mode == NETMODEUDP) || (mode == NETMODETCP))
		free(hostname);
	
	// Always free filename
	free(filename);
	
	// Everything is ok
	return 0;
}

// Send string to server
int ConnectWithString(char *hostname, unsigned int portnum, char *string, char *proto)
{
	// Socket handle
	WSADATA wsda;
	
	// Socket file descriptor
	int sockfd;

	// host entrie
	struct hostent *h; 

	// Server struct
	struct sockaddr_in server; 

	// Return value
	int ret;

	// Initialize socket
	WSAStartup(0x0101, &wsda);

	// Open a socket
	// Create tcp socket
	if(!strcmp(proto, "TCP"))
	{
		if((sockfd=socket(AF_INET,SOCK_STREAM,0))==-1)
			return RETSOCKERR;
	}

	// Create upd socket
	else if(!strcmp(proto, "UDP"))
	{
		if((sockfd=socket(AF_INET,SOCK_DGRAM,0))==-1)
			return RETSOCKERR;
	}
	
	// Cannot create socket if anything fails
	else
		return RETSOCKERR;
		
	// Resolve host
	if((h=gethostbyname(hostname)) == NULL)
			return RETRESVERR;

	// Init server struct
	server.sin_addr=*((struct in_addr*)h->h_addr);
	server.sin_port=htons(portnum);
	server.sin_family=AF_INET;
	
	// Connect with server
	if(connect(sockfd, (struct sockaddr*)&server, sizeof(struct sockaddr)) == -1)
		return RETCONNERR;

	// Send string
	ret = send(sockfd, string, strlen(string), 0);

	// Check for socket error	
	if(ret == SOCKET_ERROR)
		return RETSOCKERR;

	// Cleanup socket
	WSACleanup();
	closesocket(sockfd);

	// Everything OK
	return RETOK;
}

// Print network error strings
void PrintCheckRetVal(int ret)
{
	// Print generic error string
	fprintf(stderr, PARAMERRSTR);
	
	// Print specified error string
	switch(ret)
	{
		// Connection error
		case RETCONNERR:	fprintf(stderr, CONNERRSTR);
									break;
		// Return for socket error
		case RETSOCKERR:	fprintf(stderr, SOCKERRSTR);
									break;		
		// Error code for cannot resolve host
		case RETRESVERR:	fprintf(stderr, RESVERRSTR);
									break;
		// Wrong port number
		case RETPORTERR:	fprintf(stderr, PORTERRSTR);
									break;
		// Cannot find file
		case RETFILEERR:	fprintf(stderr, FERRSTR);
									break;
		// Unknown error code
		default:			fprintf(stderr, UNKERRSTR);
									break;
	}
}

// Print GenStrings error codes
void PrintStringsRetVal(int ret)
{
	// Print generic request error
	fprintf(stderr, STRINGERRSTR);

	// Print special error information
	switch(ret)
	{
		// Cannot open file
		case RETFILEERR:		fprintf(stderr, FERRSTR);
									break;
		// Any other error
		default:				fprintf(stderr, UNKERRSTR);
									break;
	}
}				

// Print info about team and usage
void PrintUsage(char *argv)
{
	printf("\n      TEAM PRIESTMASTERS\n");
	printf("\nEasy to use fuzzer version %s\n\n", VERSION);
	printf("URL: http://www.priestmaster.org\n");
	printf("CONTACT: priest@priestmaster.org\n");
	printf("\nNetwork mode:\n");
	printf("\nUsage: %s hostname portnum filename TCP/UDP\n", argv);
	printf("\nhostname: the victim (localhost for example)\n");
	printf("\nportnum : the port number of your victim\n");
	printf("\nfilename: your configuration file (http.cfg for example)\n");
	printf("\nprotocol: UDP for udp mode and TCP for tcp mode\n");
	printf("\n---------------------------------------\n");
	printf("\nPrint mode:\n");
	printf("\nUsage: %s filename\n", argv);
	printf("\nfilename : your configuration file (http.cfg for example)\n");
}

// Check for correct parameters
int CheckParams(int mode)
{
	// File information
	FILE *file;
	
	// For socket handling
	WSADATA	wsda;
	
	// Socket filedescriptor
	int sockfd;
	
	// for hostname resolve
	struct hostent *h; 

	// Server struct
	struct sockaddr_in server; 

	// Initialize WSA for network functions
	if((mode == NETMODEUDP) || (mode == NETMODETCP))
	{	
		// Check for correct port number	
		if(portnum < 1 || portnum > 65535)
			return RETPORTERR;
		
		// Check for correct host and port
		WSAStartup(0x0101, &wsda);
	
		// Create tcp socket
		if((mode == NETMODETCP))
		{
			if((sockfd=socket(AF_INET,SOCK_STREAM,0))==-1)
				return RETSOCKERR;
		}
		
		// Create udp socket
		else if ((mode == NETMODEUDP))
		{
			if((sockfd=socket(AF_INET,SOCK_DGRAM,0))==-1)
					return RETSOCKERR;
		}
		
		// If nothing work, return socket error
		else
			return RETSOCKERR;
	
		// Check for correct hostname
		if((h=gethostbyname(hostname)) == NULL)
			return RETRESVERR;
	
		// Init server struct
		server.sin_addr=*((struct in_addr*)h->h_addr);
		server.sin_port=htons(portnum);
		server.sin_family=AF_INET;
	
		// Connect with server
		if(connect(sockfd, (struct sockaddr*)&server, sizeof(struct sockaddr)) == -1)
			return RETCONNERR;
	
		// Cleanup WSA for network functions
		WSACleanup();
	}

	// Is it possible to open the file ?
	file = fopen(filename,"r");
	
	if(file == NULL)
		return RETFILEERR;
		
	// Close file
	fclose(file);

	// Everything OK ?	
	return RETOK;
}

// Print file error codes
void PrintFileRetVal(int ret)
{
	// Print generic error
	fprintf(stderr, INVCFRRSTR);
	
	// Print file errors
	switch(ret)
	{
		case RETFILEERR:				fprintf(stderr, FERRSTR);
												break;
		case SUBCHARERR:		fprintf(stderr, SUBCHARERRSTR);
												break;
		case STARTSEQERR:		fprintf(stderr, STARTSEQERRSTR);
												break;
		case MAXSEQERR:			fprintf(stderr, MAXSEQERRSTR);
												break;
		case STEOSEQERR:		fprintf(stderr, STEOSEQERRSTR);
												break;
		case DELEMERR:			fprintf(stderr, DELEMERRSTR);
												break;
		case INVNUMERR:			fprintf(stderr, INVNUMERRSTR);
												break;
		case NETREQTERR:		fprintf(stderr, NETREQTERRSTR);
												break;
		default: 						fprintf(stderr, UNKERRSTR);
												break;
	}
}

// Read configuration from file
int ParseFile(void)
{
	FILE *file;
	char *str;		// Temporary string
	char *str2;		// Second tmp string
	char *str3;		// string pointer
	char c;				// Temporary character
	int i;				// Iterate
	
	// Open file read only mode
	file = fopen(filename, "r");
	
	// return if file error
	if(file == NULL)
		return RETFILEERR;
	
	// Memory for string
	str = malloc(MAXRSIZ);
	str2 = malloc(MAXRSIZ);
	
	// Parse to useful data
	ParseToDataComNL(file, str);
	
	// Get first tag
	for(i=0;i<strlen(SUBCHARSTR);i++)
			str2[i] = str[i];

	// terminate str2
	str2[i] = 0;
	
	// Check for SUBCHAR string
	if(strcmp(str2, SUBCHARSTR))
		return SUBCHARERR;
	
	// Get SUBCHAR character and save it
	str3 = strstr(str, "=");
	
	// Does '=' exist ?
	if(str3 == NULL)
		return DELEMERR;
	
	// Save characters in filearg.subchar
	str3++;
	
	for(i=0;*str3 != NLCHAR;str3++)
	{
		// put char to filearg
		filearg.subchar[i] = *str3;
		i++;
		
		// Test for max subchar size
		if(i == MAXSSIZ-1)
			break;
	}
	
	// null terminate
	filearg.subchar[i] = 0;
	
	// Loop to next tag
	ParseToDataComNL(file,str);
	
	// Get second tag
	for(i=0;i<strlen(STARTSEQSTR);i++)
			str2[i] = str[i];
	
	// terminate
	str2[i] = 0;
	
	// Check for correct string
	if(strcmp(str2, STARTSEQSTR))
		return STARTSEQERR;
	
	// Search for '='
	str3 = strstr(str, DECHAR);
	
	// Does '=' exist ?
	if(str3 == NULL)
		return DELEMERR;
	
	// jump to string after '='
	str3++;
	
	// Save value in filearg
	filearg.startseq = atoi(str3);
	
	// Check for valid number
	if(filearg.startseq <= 0)
		return INVNUMERR;
	
	// Loop to next tag
	ParseToDataComNL(file,str);
	
	// Get third tag
	for(i=0;i<strlen(MAXSEQSTR);i++)
		str2[i] = str[i];
		
	// terminate
	str2[i] = 0;
	
	// Check for correct string
	if(strcmp(str2, MAXSEQSTR))
		return MAXSEQERR;
	
	// Search for '='
	str3 = strstr(str, DECHAR);
	
	// Does '=' exist ?
	if(str3 == NULL)
		return DELEMERR;
	
	// Jump to string after '='
	str3++;
	
	// Convert to integer and save in filearg
	filearg.maxseq = atoi(str3);
	
	// Check for valid number
	if(filearg.maxseq <= 0)
		return INVNUMERR;
	
	// Loop to next tag
	ParseToDataComNL(file,str);
	
	// Get fourth tag
	for(i=0;i<strlen(STEOSEQSTR);i++)
		str2[i] = str[i];
		
	// terminate
	str2[i] = 0;
	
	// Check for correct string
	if(strcmp(str2, STEOSEQSTR))
		return STEOSEQERR;
		
	// Jump to string after '='
	str3 = strstr(str, DECHAR);
	
	// Does '=' exist ?
	if(str3 == NULL)
		return DELEMERR;
	
	str3++;
	
	// Convert to integer and save in filearg
	filearg.steoseq = atoi(str3);
	
	// Check for valid number
	if(filearg.steoseq <= 0)
		return INVNUMERR;
	
	// Loop to next tag
	ParseToDataComNL(file,str);
	
	// Get fifth tag
	for(i=0;i<strlen(NETREQTSTR);i++)
		str2[i] = str[i];
		
	// terminate
	str2[i] = 0;
	
	// Check for correct string
	if(strcmp(str2, NETREQTSTR))
		return NETREQTERR;
		
	// Jump to string after '='
	str3 = strstr(str, DECHAR);
	
	// Does '=' exist ?
	if(str3 == NULL)
		return DELEMERR;
		
	str3++;
	
	// Convert to integer and save in filearg
	filearg.netreqt = atoi(str3);
	
	// Check for valid number
	if(filearg.netreqt <= 0)
		return INVNUMERR;
	
	// DEBUG DEBUG DEBUG //
	//printf("\nCHARS %s\nSTARTSEQ %d\nMAXSEQ %d\nSTEOSEQ %d\nNETREQT %d\n"
	//		   "\n", filearg.subchar, filearg.startseq, filearg.maxseq, filearg.steoseq, filearg.netreqt);
	// DEBUG DEBUG DEBUG //
	
	// free strings
	free(str);
	free(str2);
	
	// close file
	fclose(file);
	
	// Everything is ok
	return RETOK;
}

// Overstep comments and newline
int ParseToDataComNL(FILE *file, char *str)
{
	// Stop to loop, if useful data is reached
	for(;;)
	{
		// Read line and check for comments or newline
		str = fgets(str, MAXRSIZ-1, file);
		
		// End of file reached ?
		if(feof(file))
			return ENDOFFERR;
		
		// Overstep command and newline
		if((str[0] == COMCHAR) || (str[0] == NLCHAR))
			continue;
		else
			break;
	}	

	// Everything ok ?
	return RETOK;
}

// Overstep commends only
int ParseToDataCom(FILE *file, char *str)
{
	// Stop to loop, if useful data is reached
	for(;;)
	{
		// Read line and check for comments or newline
		str = fgets(str, MAXRSIZ-1, file);
		
		// End of file reached ?
		if(feof(file))
			return ENDOFFERR;
		
		// Overstep comment
		if((str[0] == COMCHAR))
			continue;
		else
			break;
	}	
	
	// Everything ok ?
	return RETOK;
}

// Generate and send or print/send strings
int GenStrings(void)
{
	// Vars
	char *str;	// Temporary string
	char *sstr;	// Start string
	char *estr; // End string
	char *fstr; // Full request
	char *p;		// simple pointer
	FILE *file; // file handle
	int i,k;		// iterate vars
	int ret;		// Return value
	
	// Open file and parse to string data
	file = fopen(filename,"r");
	
	// Possible to open file ?
	if(file == NULL)
		return RETFILEERR;	
		
	// Generic string to play with :-)
	str = malloc(MAXRSIZ);
	sstr = malloc(MAXRSIZ);
	estr = malloc(MAXRSIZ);
	fstr = malloc(MAXRSIZ);
	
	// Parse to next useful string
	// Overlook previous parsed data 6 times
	for(i=0;i<MAXFOPTS;i++)
		ParseToDataComNL(file, str);

	// Parse to next malf-string
	ParseToDataComNL(file, str);

	// Main parse loop
	while(!feof(file))
	{
		// Zeroing vars
		memset(fstr, 0x00, MAXRSIZ);
		memset(sstr, 0x00, MAXRSIZ);
		memset(estr, 0x00, MAXRSIZ);
			
		// Copy full request to str array
		while(!strstr(str, ENDMSTR))
		{	
			// Read complete request line by line and save it to fstr
			strncat(fstr, str, MAXRSIZ);
		
			// Get next line
			ret = ParseToDataCom(file, str);	
			
			// End loop
			if(ret == ENDOFFERR)
				break;
		}
	
		// Copy to full string to start string
		memcpy(sstr, fstr, MAXRSIZ);
		
		// Search for "MALFORMED"
		p = strstr(sstr, MALFSTR);
		
		// If no "MALFORMED" exist, print/send request
		if(!p)
		{
			// if netmode send string
			// for tcp
			if(mode == NETMODETCP)
			{
				sleep(filearg.netreqt);
				ret = ConnectWithString(hostname, portnum, fstr, "TCP");
			
				// Possible bug if not RETOK, because remote service is crashed
				// The {} are very important, because macro is greater than a line.
				if(ret != RETOK)
				{
					BUGOUTPUT();
				}
			}
			
			// for udp mode
			else if(mode == NETMODEUDP)
			{
				sleep(filearg.netreqt);
				ConnectWithString(hostname, portnum, fstr, "UDP");
			}
			
			// Always print string to stdout
			printf("%s", fstr);
		}
		
		// "MALFORMED" exist,
		else
		{
			// Null terminate sstr. Result = string before "MALFORMED"
			*p = 0;
			
			// Copy rest of string after malformed to end string (estr)
			strncpy(estr, p+strlen(MALFSTR), MAXRSIZ);
			
			// Main loop for the creation of malformed string.
			for(i=filearg.startseq;i<filearg.maxseq;i+=filearg.steoseq)
			{
				// use fstr to build the data
				memset(fstr, 0x00, MAXRSIZ);
				
				// Add start string sstr to request
				strncat(fstr, sstr, MAXRSIZ);
			
				// Generate subchar strings with different length
				for(k=0;k<i;k++)
					strncat(fstr, filearg.subchar, MAXRSIZ);
				
				// Concatenate estr to request
				strncat(fstr, estr, MAXRSIZ);
				
				// Send or print the data
				if(mode == NETMODETCP)
				{
					// TCP Connect
					sleep(filearg.netreqt);
					ret = ConnectWithString(hostname, portnum, fstr, "TCP");
					
					if(ret != RETOK)
					{
						// The {} are very important, because macro is greater than a line.
						BUGOUTPUT();
					}
				}
				
				// Send string over udp
				else if (mode == NETMODEUDP)
				{
					sleep(filearg.netreqt);
					ConnectWithString(hostname, portnum, fstr, "UDP");
				}
				
				// Print the string to stdout
				printf("%s", fstr);
			}
		}
		
		// End of file reached ? exit loop
		if(feof(file))
			break;

		// Read next line and overstep comments			
		ParseToDataCom(file, str);
	}

	// close file
	fclose(file);
	
	// Free unused data
	free(str);
	free(sstr);
	free(estr);
	free(fstr);
	
	// Everything is ok.
	return RETOK;
}